package com.liangjian.dataplatform.member.service;

import com.liangjian.dataplatform.member.entity.response.ResultObject;

import java.io.IOException;
import java.util.List;
import java.util.Map;

public interface IMemberextsService {


    Map<String, Object> memberextsByPhone(String data);

    Map<String, Object> index(String data);

    Map<String, Object> tagByMemberPhone(String data);


    ResultObject<List<Map<String, Object>>> findTagsByMemberPhone(String phone) throws IOException;

    ResultObject<List<Map<String, Object>>> findTagsByMemberSecretPhone(String secretPhone) throws IOException;

    ResultObject<Map<String, Object>> findBaseInfoByMemberPhone(String secretPhone) throws IOException;

    ResultObject<Map<String, Object>> findBaseInfoByMemberSecretPhone(String secretPhone) throws IOException;

}
